@@ -4,6 +4,7 @@ import logging
 import os
 import sys
 import textwrap
+import warnings
 
 from . import __version__
 from .config import (
@@ -14,6 +15,8 @@ from .config import (
     PACKAGE_PATH,
     REFERENCES_MAPPING,
     set_dry,
+    set_bustools_binary_path,
+    set_kallisto_binary_path,
     TECHNOLOGIES,
     TEMP_DIR,
 )
@@ -24,6 +27,7 @@ from .utils import (
     get_bustools_version,
     get_kallisto_version,
     make_directory,
+    open_as_text,
     remove_directory,
 )
 
@@ -134,30 +138,39 @@ def parse_ref(parser, args, temp_dir='tmp'):
             overwrite=args.overwrite,
             temp_dir=temp_dir
         )
-    elif args.workflow == 'kite':
-        ref_kite(
-            args.feature,
-            args.f1,
-            args.i,
-            args.g,
-            n=args.n,
-            k=args.k,
-            no_mismatches=args.no_mismatches,
-            overwrite=args.overwrite,
-            temp_dir=temp_dir
-        )
     else:
-        ref(
-            args.fasta,
-            args.gtf,
-            args.f1,
-            args.i,
-            args.g,
-            n=args.n,
-            k=args.k,
-            overwrite=args.overwrite,
-            temp_dir=temp_dir
-        )
+        # Report extraneous options
+        velocity_only = ['f2', 'c1', 'c2', 'flank']
+        for arg in velocity_only:
+            if getattr(args, arg):
+                parser.error(
+                    f'Option `{arg}` is not supported for workflow `{args.workflow}`'
+                )
+
+        if args.workflow == 'kite':
+            ref_kite(
+                args.feature,
+                args.f1,
+                args.i,
+                args.g,
+                n=args.n,
+                k=args.k,
+                no_mismatches=args.no_mismatches,
+                overwrite=args.overwrite,
+                temp_dir=temp_dir
+            )
+        else:
+            ref(
+                args.fasta,
+                args.gtf,
+                args.f1,
+                args.i,
+                args.g,
+                n=args.n,
+                k=args.k,
+                overwrite=args.overwrite,
+                temp_dir=temp_dir
+            )
 
 
 def parse_count(parser, args, temp_dir='tmp'):
@@ -189,7 +202,7 @@ def parse_count(parser, args, temp_dir='tmp'):
         # Smartseq can not be used with lamanno or nucleus.
         if args.x.upper() == 'SMARTSEQ':
             parser.error(
-                f'Technology {args.x} can not be used with workflow {args.workflow}.'
+                f'Technology `{args.x}` can not be used with workflow {args.workflow}.'
             )
 
         count_velocity(
@@ -224,7 +237,7 @@ def parse_count(parser, args, temp_dir='tmp'):
         # Smart-seq
         if args.x.upper() == 'SMARTSEQ':
             if args.dry_run:
-                parser.error(f'Technology {args.x} does not support dry run.')
+                parser.error(f'Technology `{args.x}` does not support dry run.')
 
             # Check for ignored arguments. (i.e. arguments either not supported or
             # not yet implemented)
@@ -234,23 +247,62 @@ def parse_count(parser, args, temp_dir='tmp'):
             for arg in ignored:
                 if getattr(args, arg):
                     logger.warning(
-                        f'Argument `{arg}` is not supported for technology {args.x}. This argument will be ignored.'
+                        f'Argument `{arg}` is not supported for technology `{args.x}`. This argument will be ignored.'
                     )
 
-            # Allow glob notation for fastqs.
-            fastqs = []
-            for expr in args.fastqs:
-                fastqs.extend(glob.glob(expr))
-            args.fastqs = sorted(list(set(fastqs)))
+            # Check if batch TSV was provided.
+            batch_path = None
+            if len(args.fastqs) == 1:
+                try:
+                    with open_as_text(args.fastqs[0], 'r') as f:
+                        if not f.readline().startswith('@'):
+                            batch_path = args.fastqs[0]
+                except Exception:
+                    pass
+
+            cells = {}
+            if batch_path:
+                with open(batch_path, 'r') as f:
+                    for line in f:
+                        if line.isspace() or line.startswith('#'):
+                            continue
+                        cell_id, fastq_1, fastq_2 = line.strip().split('\t')
+                        if cell_id in cells:
+                            parser.error(
+                                f'Found duplicate cell ID {cell_id} in {batch_path}.'
+                            )
+                        cells[cell_id] = (fastq_1, fastq_2)
+            else:
+                # Allow glob notation for fastqs.
+                fastqs = []
+                for expr in args.fastqs:
+                    fastqs.extend(glob.glob(expr))
+                if len(fastqs) % 2:
+                    logger.warning(f'{len(fastqs)} FASTQs found. ')
+                fastqs = sorted(set(fastqs))
+                for cell_id, i in enumerate(range(0, len(fastqs), 2)):
+                    fastq_1, fastq_2 = fastqs[i], (
+                        fastqs[i + 1] if i + 1 < len(fastqs) else ''
+                    )
+                    cells[cell_id] = (fastq_1, fastq_2)
             logger.info('Found the following FASTQs:')
-            for fastq in args.fastqs:
-                logger.info(' ' * 8 + fastq)
+            fastq_pairs = []
+            cell_ids = []
+            for cell_id, (fastq_1, fastq_2) in cells.items():
+                logger.info(f'        {cell_id}    {fastq_1}  {fastq_2}')
+                cell_ids.append(cell_id)
+                fastq_pairs.append((fastq_1, fastq_2))
+                if not fastq_1 or not fastq_2:
+                    parser.error(
+                        f'Single-end reads are currently not supported with technology `{args.x}`.'
+                    )
             count_smartseq(
                 args.i,
                 args.g,
                 args.x,
                 args.o,
-                args.fastqs,
+                fastq_pairs,
+                cell_ids=cell_ids,
                 threads=args.t,
                 memory=args.m,
                 overwrite=args.overwrite,
@@ -662,7 +714,11 @@ def setup_count_args(parser, parent):
     parser_count.add_argument(
         'fastqs',
         help=(
-            'FASTQ files. Glob expressions can be used only with technology `SMARTSEQ`.'
+            'FASTQ files. For technology `SMARTSEQ`, all input FASTQs are '
+            'alphabetically sorted by path and paired in order, and cell IDs '
+            'are assigned as incrementing integers starting from zero. A single '
+            'batch TSV with cell ID, read 1, and read 2 as columns can be '
+            'provided to override this behavior.'
         ),
         nargs='+'
     )
@@ -672,6 +728,10 @@ def setup_count_args(parser, parent):
 def main():
     """Command-line entrypoint.
     """
+    # Get prepackaged kallisto and bustools paths.
+    kallisto_path = get_kallisto_binary_path()
+    bustools_path = get_bustools_binary_path()
+
     # Main parser
     parser = argparse.ArgumentParser(
         description='kb_python {}'.format(__version__)
@@ -703,6 +763,18 @@ def main():
     parent.add_argument(
         '--verbose', help='Print debugging information', action='store_true'
     )
+    parent.add_argument(
+        '--kallisto',
+        help=f'Path to kallisto binary to use (default: {kallisto_path})',
+        type=str,
+        default=kallisto_path
+    )
+    parent.add_argument(
+        '--bustools',
+        help=f'Path to bustools binary to use (default: {bustools_path})',
+        type=str,
+        default=bustools_path
+    )
 
     # Command parsers
     setup_info_args(subparsers, argparse.ArgumentParser(add_help=False))
@@ -757,7 +829,12 @@ def main():
 
     # Turn off logging from other packages.
     try:
-        logging.getLogger('anndata').disable(level=logging.CRITICAL)
+        anndata_logger = logging.getLogger('anndata')
+        h5py_logger = logging.getLogger('h5py')
+        anndata_logger.setLevel(logging.CRITICAL + 10)
+        anndata_logger.propagate = False
+        h5py_logger.setLevel(logging.CRITICAL + 10)
+        h5py_logger.propagate = False
     except Exception:
         pass
 
@@ -769,12 +846,14 @@ def main():
         ))
 
     logger.debug('Printing verbose output')
-    logger.debug(
-        'kallisto binary located at {}'.format(get_kallisto_binary_path())
-    )
-    logger.debug(
-        'bustools binary located at {}'.format(get_bustools_binary_path())
-    )
+
+    # Set binary paths
+    set_kallisto_binary_path(args.kallisto)
+    set_bustools_binary_path(args.bustools)
+
+    logger.debug(f'kallisto binary located at {get_kallisto_binary_path()}')
+    logger.debug(f'bustools binary located at {get_bustools_binary_path()}')
+
     temp_dir = args.tmp or os.path.join(
         args.o, TEMP_DIR
     ) if 'o' in args else TEMP_DIR
@@ -782,7 +861,9 @@ def main():
     make_directory(temp_dir)
     try:
         logger.debug(args)
-        COMMAND_TO_FUNCTION[args.command](parser, args, temp_dir=temp_dir)
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            COMMAND_TO_FUNCTION[args.command](parser, args, temp_dir=temp_dir)
     except Exception:
         if is_dry():
             raise
