@@ -14,6 +14,7 @@ from urllib.request import urlretrieve
 import anndata
 import pandas as pd
 import scipy.io
+from scipy import sparse
 from tqdm import tqdm
 
 from .config import (
@@ -165,6 +166,7 @@ def run_executable(
     quiet=False,
     returncode=0,
     alias=True,
+    record=True,
 ):
     """Execute a single shell command.
 
@@ -192,6 +194,8 @@ def run_executable(
     :param alias: whether to use the basename of the first element of `command`,
                   defaults to `True`
     :type alias: bool, optional
+    :param record: whether to record the call statistics, defaults to `True`
+    :type record: bool, optional
 
     :return: the spawned process
     :rtype: subprocess.Process
@@ -202,7 +206,7 @@ def run_executable(
         c[0] = os.path.basename(c[0])
     if not quiet:
         logger.debug(' '.join(c))
-    if not wait:
+    if not wait and record:
         STATS.command(c)
     start = time.time()
     p = sp.Popen(
@@ -225,7 +229,8 @@ def run_executable(
                 for line in p.stderr:
                     out.append(line.strip())
                     logger.debug(line.strip())
-        STATS.command(c, runtime=time.time() - start)
+        if record:
+            STATS.command(c, runtime=time.time() - start)
 
         if not quiet and p.returncode != returncode:
             logger.error('\n'.join(out))
@@ -242,7 +247,10 @@ def get_kallisto_version():
     :return: tuple of major, minor, patch versions
     :rtype: tuple
     """
-    p = run_executable([get_kallisto_binary_path()], quiet=True, returncode=1)
+    p = run_executable([get_kallisto_binary_path()],
+                       quiet=True,
+                       returncode=1,
+                       record=False)
     match = VERSION_PARSER.match(p.stdout.read())
     return tuple(int(ver) for ver in match.groups()) if match else None
 
@@ -255,7 +263,10 @@ def get_bustools_version():
     :return: tuple of major, minor, patch versions
     :rtype: tuple
     """
-    p = run_executable([get_bustools_binary_path()], quiet=True, returncode=1)
+    p = run_executable([get_bustools_binary_path()],
+                       quiet=True,
+                       returncode=1,
+                       record=False)
     match = VERSION_PARSER.match(p.stdout.read())
     return tuple(int(ver) for ver in match.groups()) if match else None
 
@@ -295,7 +306,8 @@ def get_supported_technologies():
     """
     p = run_executable([get_kallisto_binary_path(), 'bus', '--list'],
                        quiet=True,
-                       returncode=1)
+                       returncode=1,
+                       record=False)
     return parse_technologies(p.stdout)
 
 
@@ -481,6 +493,34 @@ def get_temporary_filename(temp_dir=None):
     return path
 
 
+def read_t2g(t2g_path):
+    """Given a transcript-to-gene mapping path, read it into a dictionary.
+    The first column is always assumed to tbe the transcript IDs.
+
+    :param t2g_path: path to t2g
+    :type t2g_path: str
+
+    :return: dictionary containing transcript IDs as keys and all other columns
+             as a tuple as values
+    :rtype: dict
+    """
+    t2g = {}
+    with open_as_text(t2g_path, 'r') as f:
+        for line in f:
+            if line.isspace():
+                continue
+            split = line.strip().split('\t')
+            transcript = split[0]
+            other = tuple(split[1:])
+            if transcript in t2g:
+                logger.warning(
+                    f'Found duplicate entries for {transcript} in {t2g_path}. '
+                    'Earlier entries will be ignored.'
+                )
+            t2g[transcript] = other
+    return t2g
+
+
 def import_tcc_matrix_as_anndata(
     matrix_path, barcodes_path, ec_path, txnames_path, threads=8
 ):
@@ -567,23 +607,38 @@ def import_matrix_as_anndata(
     df_genes.index = df_genes.index.astype(
         str
     )  # To prevent logging from anndata
+    mtx = scipy.io.mmread(matrix_path)
+
+    # If any of the genes are duplicated, collapse them by summing
+    if any(df_genes.index.duplicated()):
+        logger.debug(
+            f'Deduplicating genes found in {genes_path} by adding duplicates'
+        )
+        mtx = mtx.tocsc()
+        gene_indices = {}
+        for i, gene in enumerate(df_genes.index):
+            gene_indices.setdefault(gene, []).append(i)
+        genes = []
+        deduplicated_mtx = sparse.lil_matrix(
+            (len(df_barcodes), len(gene_indices))
+        )
+        for i, gene in enumerate(sorted(gene_indices.keys())):
+            genes.append(gene)
+            indices = gene_indices[gene]
+            deduplicated_mtx[:, i] = mtx[:, indices].sum(axis=1)
+        df_genes = pd.DataFrame(index=pd.Series(genes, name=f'{name}_id'))
+        mtx = deduplicated_mtx
+
+    t2g = read_t2g(t2g_path) if t2g_path else {}
     id_to_name = {}
-    if t2g_path:
-        with open(t2g_path, 'r') as f:
-            for line in f:
-                if line.isspace():
-                    continue
+    for transcript, attributes in t2g.items():
+        if len(attributes) > 1:
+            id_to_name[attributes[0]] = attributes[1]
+    gene_names = [id_to_name.get(i, '') for i in df_genes.index]
+    if any(bool(g) for g in gene_names):
+        df_genes[f'{name}_name'] = gene_names
 
-                split = line.strip().split('\t')
-                if len(split) > 2:
-                    id_to_name[split[1]] = split[2]
-        gene_names = [id_to_name.get(i, '') for i in df_genes.index]
-        if any(bool(g) for g in gene_names):
-            df_genes[f'{name}_name'] = gene_names
-
-    return anndata.AnnData(
-        X=scipy.io.mmread(matrix_path).tocsr(), obs=df_barcodes, var=df_genes
-    )
+    return anndata.AnnData(X=mtx.tocsr(), obs=df_barcodes, var=df_genes)
 
 
 def overlay_anndatas(adata_spliced, adata_unspliced):
@@ -606,6 +661,7 @@ def overlay_anndatas(adata_spliced, adata_unspliced):
     df_obs = unspliced_intersection.obs
     df_var = unspliced_intersection.var
     return anndata.AnnData(
+        X=spliced_intersection.X,
         layers={
             'spliced': spliced_intersection.X,
             'unspliced': unspliced_intersection.X
